/*******************************************************************************

 BURGLAR_SMS_ALERT_V2

 
 Alerta de disparo de alarme via SMS 
 
 C/ Pic 12f675 e mdulo SIM800L

 Autor: Cludio Lrios

 Data: 04/04/2019
 
 Objetivo: Enviar SMS para o dono do veculo quando o seu alarme disparar.
 Trabalha em conjunto com o alarme do carro, somente por uma simples conexo na
 buzina ou sirene. 
 (Nota: Isto no  um tipo de alarme de carro! Apenas um complemento a ele!)

 Como Programar o Nmero do Celular que ser alertado por SMS:
 1) Certifique que no esteja ocorrendo uma situao de disparo.
 2) Ligue o dispositivo e aguarde o led do mdulo SIM800L piscar a cada 3 seg.
    ( O mdulo SIM800L tem que ter um carto SIM inserido nele e funcionando). 
 3) Aperte brevemente o boto 'SALVA_N' (apenas pressione e solte). 
 4) Faa uma chamada a partir do celular que ser usado para receber a notifi-
    cao de alarme disparado. 
 5) Imediatamente, ocorrer  a rejeio da chamada pelo dispositivo.
 6) Aguarde alguns segundos a confirmao por um SMS com a mensagem:
    "BURGLAR_SMS_ALERT-> NUMERO DE CELULAR SALVO".
    
 Depois faa teste disparando o alarme. Cerca de 5 segundos aps disparo,
    o dispositivo dever enviar um SMS com  a mensagem:
   "BURGLAR_SMS_ALERT-> ATENCAO: ALARME FOI DISPARADO!".   
   

 Blog Picsource

 Uso didtico

*******************************************************************************/

#include "burglar_sms_alert_V2.h" //arquivo de cabealho
#use fast_io(a)

//bytes
#byte trisio = 0x85 //sentido dos pinos
#byte gpio   = 0x05 //gpio
#byte tmr0   = 0x01
#byte ioc    = 0x96

//bit
#bit t0if    = 0xb.2
#bit gpif    = 0xb.0
//defines
#define sw_save  PIN_A3//0x5.3  - pino 4
#define in_disp  PIN_A0//0x5.0  - pino 7
#define tx       PIN_A1//0x5.1  - pino 6
#define rx       PIN_A2//0x5.2  - pino 5

#define CEL_NUMBER_SIZE 18
#define NUM_MINIMO 7  //quantidade minima de caracteres numricos no cel_buffer
#define TEMPO_POS_DISPARO 5//aps disparo do alarme, envia sms aps este tempo
#define BUFFER_SIZE   32
//variveis globais


unsigned int8 a;
char cel_buffer[BUFFER_SIZE];//armazena numero da ltima ligao de celular

/******************************************************************************
    Rotina de Delay de 1 Segundo
 ******************************************************************************/

void delay_1s(){
  delay_ms(1000);//delay 1 segundo
}
/******************************************************************************
    Rotina de DElay para 'N' segundos
 ******************************************************************************/
void delay_s(unsigned int8 valor){
unsigned int8 a;
  for(a=0;a<valor;a++){
   delay_1s();
  }
}
/******************************************************************************
    Rotina de Escrita na EEprom Interna do Pic
 ******************************************************************************/
void write_eep(unsigned int8 address,valor){ 
  write_eeprom(address,valor); 
}
/*******************************************************************************
    Rotina de Leitura na EEprom Interna do Pic
*******************************************************************************/
unsigned int8 read_eep(unsigned address){
  return read_eeprom(address);
}
/*******************************************************************************
    Rotina de Escrita do Nmero de Celular na EEprom Interna do Pic
    
    Esta rotina comea imediatamente aps ser apertado o boto 'SALVE_N".
    O modulo Sim800L  ir enviar a seguinte mensagem ao receber um ligao
    (isto se anteriormente foi enviado a ativao de identificao de chamadas
    usando o comando 'AT+CLIP'):
    
     RING
    +CLIP: "xxxxxxxxxxxxxxx",161,"",0,"",0
   
    (sendo x os dgitos do nmero do telefone celular que ligou)
    
    Esta rotina ir captura no buffer os 30 primeiros caracteres e extrair
    deles o nmero do celular que ligou. O critrio da extrao so as aspas
    iniciais e finais (a primeira ocorrncia tem o nmero do celular).
*******************************************************************************/

void salva_numero_EEp(){//salva numero do celular que ligou na eeprom  
  unsigned int8 i,j,d=0;
  unsigned int1 ft1=0;
       
  for(i=0;i<BUFFER_SIZE;i++){     //captura bytes da serial (rx) no buffer
      cel_buffer[i]=getc();     
  }
     
     //localiza nmero no buffer e salva na eeprom interna
     
  for(i=0;i<BUFFER_SIZE;i++){
      j=cel_buffer[i];
      
      if(j==0x22 && ft1){//se for 2 aspas (") fim do salvamento
         write_eep(d,0x00);//byte de finalizao = 0
         break;
         }
      if(j==0x22 && !ft1){//se for 1 aspas, prepara para captura nmero
         ft1=1;
         continue;
         }

      if(!ft1)continue;
      
      if(j>='0'&& j<='9') {//salva apenas os numeros entre aspas
        write_eep(d++,j);
      }
  }//for(i=0;i<BUFFER_SIZE;i++)
  
  printf("ATH\r\n");//rejeita chamada 
  delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       
}//void salva_numero_EEp()
 /******************************************************************************
    Rotina de Envio de SMS p/ Confirmar que o Nmero do Celular foi Salvo
    (o SMS ir ser enviado para o mesmo nmero salvo)
 ******************************************************************************/
void env_sms_confirmacao(){
  char val;
 /******************************************************************************
    Informa ao mdulo qual celular ser enviado a confirmao do comando
 ******************************************************************************/
       printf("AT+CMGF=1\r\n");//modo texto
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       //comando para enviar numero do celular
       printf("AT+CMGS=\"");//envia aspas iniciais com cdigo de barra invertida \"
       for(a=0;a<CEL_NUMBER_SIZE;a++){
          val=read_eep(a);
          if(val>'9'||val<'0')break;//fim dos numeros
          printf("%c", val);//envia dgitos do numero do celular salvo na EEprom
       }
       printf("\"\r\n");//envia aspas finais e 'enter'

       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       printf("BURGLAR_SMS_ALERT-> NUMERO DE CELULAR SALVO");//mensagem
       printf("\x1A"); //envia control z
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
    

}//env_sms_confirmacao()

 /******************************************************************************
    Rotina de Envio de Mensagem de Alerta por Disparo de Alarme
 ******************************************************************************/
void env_sms_alerta(){
  char val;

       printf("AT+CMGF=1\r\n");//modo texto
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       //comando para enviar numero do celular
       printf("AT+CMGS=\"");//envia aspas iniciais com cdigo de barra invertida \"
       for(a=0;a<CEL_NUMBER_SIZE;a++){
          val=read_eep(a);
          if(val>'9'||val<'0')break;//fim dos numeros
          printf("%c", val);//envia dgitos do numero do celular salvo na EEprom
       }
       printf("\"\r\n");//envia aspas finais e 'enter'

       delay_1s();//aguarda receber 'ok' antes de enviar outro comando
       printf("BURGLAR_SMS_ALERT-> ATENCAO: ALARME FOI DISPARADO!");//mensagem
       printf("\x1A"); //envia control z
       delay_1s();//aguarda receber 'ok' antes de enviar outro comando

}

/******************************************************************************
     Rotina Main - Configuraes dos Registradores
*******************************************************************************/


void main() {
#define TEMPO_MAX 2000000

   setup_adc_ports(NO_ANALOGS);
   setup_adc(ADC_OFF);
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   trisio= 0b111101;
   delay_s(20);//delay de 20 segundos para o mdulo SIM800L se estabilizar
   printf("AT\r\n");//envia comando apenas para sincronizar baudrate
   delay_1s();//aguarde
   printf("ATE0\r\n");//envia comando p/ no ecoar comandos
   delay_1s();//aguarde
   printf("AT+CMGF=1\r\n");//envia comando para modo texto
   delay_1s();//aguarde
   printf("AT+CLIP=1\r\n");//envia comando de identifio de chamadas
   delay_1s();//aguarde

/******************************************************************************
     Loop Continuo
*******************************************************************************/

   for(;;){

/*******************************************************************************
   Rotina para salvamento de numero do celular.
   Atravs do comando de identifio de chamadas (at+clip=1) ser visto o
   nmero que liga a cada "ring".
   Aperte 'salva_n' e ligue para o numero do carto Sim inserido com o celular
   que ir receber o SMS. Aps o salvamento, receber SMS de confirmao.
*******************************************************************************/

     if(!input(sw_save)){// se interruptor for pressionado 
        salva_numero_EEp();//captura e salva n.cel na eeprom
        env_sms_confirmacao(); //envia sms de confirmao do salvamento de n.cel
        if(!input(sw_save))delay_ms(100);//aguarda solta boto
     }// if(!input(sw_save))...

/******************************************************************************
       Rotina de verificao de Disparo e Envio de SMS
*******************************************************************************/

    if(input(in_disp)){
     //se for disparado pega da eeprom o nmero e envia SMS
       delay_s(TEMPO_POS_DISPARO);//p/ no acionar quando liga e desliga alarme
        if(input(in_disp)){//confere situao atual do pino de disparo
         env_sms_alerta();//envia SMS de alerta
         while(input(in_disp))delay_s(1);//aguarda por desligamento do alarme
       }// if(!in_disp)
    }// if(!in_disp)
   }//for(;;)
}//main()

//=============================fim==============================================
